#include "slist.h"
#include <stdlib.h>
#include <assert.h>

struct List *insert_front(struct List *head, int item)
/* Add new item to the head of the list. Return pointer to the new head. */
{
    /* Allocate memory for new node. */
    struct List *newnode = (struct List *) malloc(sizeof(struct List));
    if (newnode == NULL)
        return NULL; /* allocation failed */

    /* Attach new node in front of head */
    newnode->item = item;
    newnode->next = head;
    return newnode;
}

struct List *insert_back(struct List *node, int item)
/* Add new item to the end of the list. Return pointer to the new List node. */
{
    /* Allocate memory for new node. */
    struct List *newnode = (struct List *) malloc(sizeof(struct List));
    if (newnode == NULL)
        return NULL; /* allocation failed */

    newnode->item = item;
    newnode->next = NULL;

    /* If list is not empty, find end of list and attach new node. */
    if (node) {
        while (node->next)
            node = node->next;
        node->next = newnode;
    }

    return newnode;
}

struct List *insert_after(struct List *node, int item)
/* Add new item to the next position in the list after node. */
{
    /* Allocate memory for new node. */
    struct List *newnode = (struct List *) malloc(sizeof(struct List));
    if (newnode == NULL)
        return NULL; /* allocation failed */

    /* If list is not empty, splice new node into list. */
    if (node) {
        newnode->next = node->next;
        node->next = newnode;
    }
    else 
        newnode->next = NULL;

    newnode->item = item;
    return newnode;
}

struct List *delete_node(struct List *head, struct List *node)
/* Delete the specified node. Return the node before it, unless the 
 * specified node is the head, then return the new head. */
{
    assert(head != NULL && node != NULL);

    if (node == head) { /* delete old head and return new head */
        head = head->next;
        free(node);
        return head; 
    }
    
    while (head->next != node) /* search for node */
        head = head->next;

    return delete_after(head);
}

struct List *delete_after(struct List *node)
/* Delete the node AFTER the specified node. Return the specified node. */
{
    struct List *tmp;
    assert(node != NULL && node->next != NULL);

    tmp = node->next;
    node->next = tmp->next; /* reconnect list around deleted node */
    free(tmp);

    return node;
}

void delete_all(struct List *head)
/* Delete entire list, from head to tail. */
{
    while (head) {
        struct List *tmp = head;
        head = head->next;
        free(tmp);
    }
}
